from dataclasses import dataclass
from typing import Optional

from hostingde.model import Model


@dataclass
class DeleteZoneRequest(Model):

    zone_config_id: Optional[str]
    zone_name: Optional[str]



from typing import Optional, List

from hostingde.dns.requests.update_zone_request import UpdateZoneRequest
from hostingde.exceptions import ClientException
from hostingde.job_waiter import AsynchronousClient, JobWaiter
from hostingde.model.zone_config import ZoneConfig

from hostingde.dns.requests.delete_zone import DeleteZoneRequest
from hostingde.dns.requests.create_new_zone import CreateZoneRequest
from hostingde.hostingde import HostingDeCore
from hostingde.model.filter import FilterElement
from hostingde.model.job import Job
from hostingde.model.sort import SortConfiguration
from hostingde.model.zone import Zone
from hostingde.paginator import HostingDePaginator
from hostingde.model.record import Record


class DnsClient(HostingDeCore, AsynchronousClient):

    def __init__(self, parent: HostingDeCore):
        """
        Construct a new client

        :param parent: The parent to retrieve the session from
        """
        super().__init__(parent)

    def build_uri(self, method: str):
        """
        Utility method to easily construct URLs for this service.

        :param method: The method within this service
        :return: The URL string
        """
        return self._build_uri('dns', method)

    def list_zones(self,
                   limit: Optional[int] = None,
                   filter: Optional[FilterElement] = None,
                   sort: Optional[SortConfiguration] = None,
                   *args, **kwargs) -> HostingDePaginator[Zone]:
        """
        Retrieves a list of Zone objects from the generic filtering and sorting API.

        +-------------------------+-------------------------------------------------------+
        | Field                   | Description                                           |
        +-------------------------+-------------------------------------------------------+
        | AccountId               | ID of the account to which the zone belongs           |
        +-------------------------+-------------------------------------------------------+
        | ZoneConfigId            | ID of the zoneConfig object                           |
        +-------------------------+-------------------------------------------------------+
        | ZoneName                | Zone name in ACE format                               |
        +-------------------------+-------------------------------------------------------+
        | ZoneNameUnicode         | Zone name in Unicode format                           |
        +-------------------------+-------------------------------------------------------+
        | ZoneMasterIp            | Master IP address of the zone                         |
        +-------------------------+-------------------------------------------------------+
        | ZoneType                | Zone type (“MASTER”, “NATIVE”, or “SLAVE”)            |
        +-------------------------+-------------------------------------------------------+
        | ZoneEmailAddress        | Email address of the zone                             |
        +-------------------------+-------------------------------------------------------+
        | ZoneLastChangeDate      | Time and date of last modification                    |
        +-------------------------+-------------------------------------------------------+
        | ZoneSoaRefresh          | SOA refresh interval of slaves (seconds)              |
        +-------------------------+-------------------------------------------------------+
        | ZoneSoaRetry            | SOA retry interval of slaves (seconds)                |
        +-------------------------+-------------------------------------------------------+
        | ZoneSoaExpire           | SOA deactivation timeout of slaves (seconds)          |
        +-------------------------+-------------------------------------------------------+
        | ZoneSoaTtl              | SOA time to live for negative caching (seconds)       |
        +-------------------------+-------------------------------------------------------+
        | ZoneIpv4Replacement     | Replacement value for IPv4 addresses in template      |
        +-------------------------+-------------------------------------------------------+
        | ZoneIpv6Replacement     | Replacement value for IPv6 addresses in template      |
        +-------------------------+-------------------------------------------------------+
        | ZoneMailIpv4Replacement | Replacement value for IPv4 mail addresses in template |
        +-------------------------+-------------------------------------------------------+
        | ZoneMailIpv6Replacement | Replacement value for IPv6 mail addresses in template |
        +-------------------------+-------------------------------------------------------+
        | TemplateName            | Name of template associated with zone                 |
        +-------------------------+-------------------------------------------------------+
        | TemplateId              | ID of template associated with zone                   |
        +-------------------------+-------------------------------------------------------+
        | RecordId                | ID of record in zone                                  |
        +-------------------------+-------------------------------------------------------+
        | RecordName              | Name of record in zone                                |
        +-------------------------+-------------------------------------------------------+
        | RecordType              | Type of record in zone                                |
        +-------------------------+-------------------------------------------------------+
        | RecordContent           | Content of record in zone                             |
        +-------------------------+-------------------------------------------------------+
        | RecordTtl               | Time to live (in seconds)                             |
        +-------------------------+-------------------------------------------------------+
        | RecordPriority          | Priority of record in zone (integer)                  |
        +-------------------------+-------------------------------------------------------+
        | RecordLastChangedate    | Date and time of last modification in zone            |
        +-------------------------+-------------------------------------------------------+

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields Zone objects.
        """

        uri = self._build_uri('dns', 'zonesFind')

        return self._iter(uri, Zone, filter, limit, sort)

    def list_zone_configs(self,
                          limit: Optional[int] = None,
                          filter: Optional[FilterElement] = None,
                          sort: Optional[SortConfiguration] = None,
                          *args, **kwargs) -> HostingDePaginator[Zone]:
        """
        Retrieves a list of ZoneConfig objects from the generic filtering and sorting API.

        +-------------------------+-----------------------------------------------------------+
        | Field                   | Description                                               |
        +-------------------------+-----------------------------------------------------------+
        | AccountId               | ID of the account that the zone belongs to                |
        +-------------------------+-----------------------------------------------------------+
        | ZoneConfigId            | ZoneConfig object ID                                      |
        +-------------------------+-----------------------------------------------------------+
        | ZoneName                | Zone name in ACE format                                   |
        +-------------------------+-----------------------------------------------------------+
        | ZoneNameUnicode         | Zone name in Unicode format                               |
        +-------------------------+-----------------------------------------------------------+
        | ZoneMasterIp            | Master IP address of the zone                             |
        +-------------------------+-----------------------------------------------------------+
        | ZoneType                | Zone type (“MASTER”, “NATIVE”, or “SLAVE”)                |
        +-------------------------+-----------------------------------------------------------+
        | ZoneEmailAddress        | Email address of hostmaster                               |
        +-------------------------+-----------------------------------------------------------+
        | ZoneLastChangeDate      | Date and time of last zone update                         |
        +-------------------------+-----------------------------------------------------------+
        | ZoneSoaRefresh          | SOA refresh interval of slaves (seconds)                  |
        +-------------------------+-----------------------------------------------------------+
        | ZoneSoaRetry            | SOA retry interval of slaves (seconds)                    |
        +-------------------------+-----------------------------------------------------------+
        | ZoneSoaExpire           | SOA deactivation timeout of slaves (seconds)              |
        +-------------------------+-----------------------------------------------------------+
        | ZoneSoaTtl              | SOA time to live for negative caching (seconds)           |
        +-------------------------+-----------------------------------------------------------+
        | ZoneIpv4Replacement     | Replacement value for ##IPv4## placeholder in template    |
        +-------------------------+-----------------------------------------------------------+
        | ZoneIpv6Replacement     | Replacement value for ##IPv6## placeholder in template    |
        +-------------------------+-----------------------------------------------------------+
        | ZoneMailIpv4Replacement | Replacement value for ##IPv4## mail addresses in template |
        +-------------------------+-----------------------------------------------------------+
        | ZoneMailIpv6Replacement | Replacement value for ##IPv6## mail addresses in template |
        +-------------------------+-----------------------------------------------------------+
        | TemplateName            | Name of template associated with zone                     |
        +-------------------------+-----------------------------------------------------------+
        | TemplateId              | ID of template associated with zone                       |
        +-------------------------+-----------------------------------------------------------+

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields Zone objects.
        """

        uri = self._build_uri('dns', 'zoneConfigsFind')

        return self._iter(uri, ZoneConfig, filter, limit, sort)

    def list_records(self,
                     limit: Optional[int] = None,
                     filter: Optional[FilterElement] = None,
                     sort: Optional[SortConfiguration] = None,
                     *args, **kwargs) -> HostingDePaginator[Record]:
        """
        Retrieves a list of Record objects from the generic filtering and sorting API.

        +----------------------+---------------------------------------------------------+
        | Field                | Description                                             |
        +----------------------+---------------------------------------------------------+
        | ZoneConfigId         | ID of the zoneConfig object to which the record belongs |
        +----------------------+---------------------------------------------------------+
        | RecordId             | Record object ID                                        |
        +----------------------+---------------------------------------------------------+
        | RecordName           | Name of the record                                      |
        +----------------------+---------------------------------------------------------+
        | RecordType           | Type of the record                                      |
        +----------------------+---------------------------------------------------------+
        | RecordContent        | Content of the record                                   |
        +----------------------+---------------------------------------------------------+
        | RecordTtl            | Time to live (in seconds)                               |
        +----------------------+---------------------------------------------------------+
        | RecordPriority       | Priority of the record (integer)                        |
        +----------------------+---------------------------------------------------------+
        | RecordLastChangeDate | Date and time of last modification                      |
        +----------------------+---------------------------------------------------------+

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields Zone objects.
        """

        uri = self._build_uri('dns', 'recordsFind')

        return self._iter(uri, Record, filter, limit, sort)

    def delete_zone(self, zone_config_id: Optional[str] = None, zone_name: Optional[str] = None,
                    asynchronous: bool = None) -> bool:
        """
        The complete zone, ie. the zoneConfig and all records, will be deleted.
        Either the ID or the name has to be provided. If both are set, an error will be returned.

        :param zone_config_id: The ID of the zone to delete
        :param zone_name: The name of the zone to delete
        :param asynchronous: Perform delete async. Synchronous is only supported for a given zone_config_id.
        :return: EmptyResponse
        """

        if zone_config_id is None and zone_name is None:
            raise ClientException('At least one parameter has to be provided.')

        if zone_config_id is not None and zone_name is not None:
            raise ClientException('Only one parameter must be provided: Either the name, or the id, but not both.')

        if zone_name and not asynchronous:
            raise ClientException('Deleting a zone synchronously by name is currently not supported!')

        url = self.build_uri('zoneDelete')
        self._request(url, DeleteZoneRequest(
            zone_config_id=zone_config_id,
            zone_name=zone_name
        ))

        if not asynchronous and zone_config_id is not None:
            JobWaiter(self, zone_config_id).wait()

        return True

    def jobs_find(self,
                  limit: Optional[int] = None,
                  filter: Optional[FilterElement] = None,
                  sort: Optional[SortConfiguration] = None,
                  *args, **kwargs) -> HostingDePaginator[Job]:
        """
        Retrieves a list of Job objects from the generic filtering and sorting API.

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields ZoneConfig objects.
        """
        uri = self._build_uri('dns', 'jobsFind')

        return self._iter(uri, Job, filter, limit, sort)

    def update_zone(self,
                    zone_config: ZoneConfig,
                    records_to_add: Optional[List[Record]] = None,
                    records_to_delete: Optional[List[Record]] = None,
                    records_to_modify: Optional[List[Record]] = None,
                    asynchronous: Optional[bool] = None) -> Zone:
        """
        You can use zoneUpdate to make adjustments to the zone's zoneConfig, to remove records, to add new records or
        to modify existing records.

        All records in recordsToAdd will be added to the zone, while all records in recordsToDelete will be deleted.
        All records in recordsToModify will be modified. If you insert a record that does not exist into recordsToDelete
        or recordsToModify, an error will occur.

        Existing records that are not contained in either list will not be changed.

        :param zone_config: zoneConfig to be updated
        :param records_to_add: Records to be added
        :param records_to_delete: Records to be modified
        :param records_to_modify: Records to be deleted
        :param asynchronous: Update the zone asynchronously. If not provided, defaults to synchronous mode.
        :return:
        """
        url = self.build_uri('zoneUpdate')

        response = self._request(url, UpdateZoneRequest(
            zone_config=zone_config,
            records_to_add=records_to_add,
            records_to_delete=records_to_delete,
            records_to_modify=records_to_modify
        ))

        if not asynchronous:
            JobWaiter(self, zone_config.id).wait()

        return self._instance(Zone, response.json().get('response', {}))

    def create_zone(self,
                    zone_config: ZoneConfig,
                    records: List = None,
                    nameserver_set_id: Optional[str] = None,
                    use_default_nameserver_set: Optional[bool] = None,
                    asynchronous: Optional[bool] = None) -> Zone:
        """
        To create a zone, you need at least a zoneConfig.

        A zone may be created manually or from a template. If you want to use a template, you have to at least provide
        a template ID in the zoneConfig's template values property. See template object for more details.

        See NameserverSet object for more details on using nameserver sets.

        :param zone_config: zoneConfig of the zone
        :param records: Records of the zone
        :param nameserver_set_id: NameserverSet to use for automatic creation of NS records. Default: 0
        :param use_default_nameserver_set: Use your account's default nameserver set. Default: false
        :param asynchronous: Create the zone asynchronously. If not provided, defaults to False (synchronous mode).
        :return:
        """
        if records is None:
            records = []

        url = self.build_uri('zoneCreate')

        response = self._request(url, CreateZoneRequest(
            zone_config=zone_config,
            records=records,
            nameserver_set_id=nameserver_set_id,
            use_default_nameserver_set=use_default_nameserver_set
        ))

        zone: Zone = self._instance(Zone, response.json().get('response', {}))

        if not asynchronous:
            JobWaiter(self, zone.zone_config.id).wait()

        return zone



from dataclasses import dataclass, field
from hostingde.model.zone_config import ZoneConfig
from hostingde.model import Model
from typing import List, Optional
from hostingde.model.record import Record


@dataclass
class CreateZoneRequest(Model):
    zone_config: ZoneConfig
    nameserver_set_id: Optional[str]
    use_default_nameserver_set: Optional[bool]
    records: List[Record] = field(default_factory=list)



from dataclasses import field, dataclass
from typing import Optional

from hostingde.model import Model


@dataclass
class SoaValues(Model):

    refresh: int = field(default=86400)
    retry: int = field(default=7200)
    expire: int = field(default=3600000)
    ttl: int = field(default=172800)
    negative_ttl: int = field(default=3600)

    def __init__(self, refresh: Optional[int] = 86400, retry: Optional[int] = 7200, expire: Optional[int] = 3600000,
                 ttl: Optional[int] = 172800, negative_ttl: Optional[int] = 3600):
        """
        The SOA values object contains the time (seconds) used in a zone's SOA record. The maximum number of seconds is
        31556926 which equals one year. Exceeding the maximum and uncercutting the minimum values will cause the request
        to be aborted and create an error.

        :param refresh: Refresh for the SOA record. Default: 86400, minimum: 3600.
        :param retry: Retry for the SOA record. Default: 7200, minimum: 600.
        :param expire: Expire for the SOA record. Default: 3600000, minimum: 86400.
        :param ttl: TTL for the SOA record. Default: 172800, minimum: 60.
        :param negative_ttl: Negative TTL for the SOA record. Default: 3600, minimum: 60.
        """
        super().__init__()
        self.refresh: int = refresh
        self.retry: int = retry
        self.expire: int = expire
        self.ttl: int = ttl
        self.negative_ttl: int = negative_ttl



class ApiException(Exception):
    """
    The client throws an API exception, whenever the error was unknown. Details are included in the exception.
    """

    def __init__(self, details: dict, *args, **kwargs):
        super().__init__(args, kwargs)
        self.details = details

    def __str__(self):
        return "\n".join([
            f"{error.get('text')}" for error in self.details.get('errors', [])
        ])


class ClientException(Exception):
    pass



from typing import Optional

from hostingde.hostingde import HostingDeCore
from hostingde.model.account import Account
from hostingde.model.filter import FilterElement
from hostingde.model.sort import SortConfiguration
from hostingde.paginator import HostingDePaginator


class AccountClient(HostingDeCore):

    def __init__(self, parent: HostingDeCore):
        """
        Construct a new Account client

        :param parent: The parent to retrieve the session from
        """
        super().__init__(parent)

    def build_uri(self, method: str):
        """
        Utility method to easily construct URLs for this service.

        :param method: The method within this service
        :return: The URL string
        """
        return self._build_uri('account', method)

    def list_subaccounts_names(self,
                               limit: Optional[int] = None,
                               filter: Optional[FilterElement] = None,
                               sort: Optional[SortConfiguration] = None,
                               *args, **kwargs) -> HostingDePaginator[Account]:
        """
        List subaccounts for the current context.

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields Zone objects.
        """

        uri = self._build_uri('account', 'subaccountsFind')

        return self._iter(uri, Account, filter, limit, sort)



import marshmallow_dataclass
from marshmallow import Schema, post_dump


def camelcase(s):
    parts = iter(s.split("_"))
    return next(parts) + "".join(i.title() for i in parts)


class CamelCaseSchema(Schema):
    """Schema that uses camel-case for its external representation
    and snake-case for its internal representation.
    """

    SKIP_VALUES = {None}

    def __init__(self, **kwargs):
        super().__init__(unknown='EXCLUDE', **kwargs)

    def on_bind_field(self, field_name, field_obj):
        """Use lower-camel-casing as external representation"""
        field_obj.data_key = camelcase(field_obj.data_key or field_name)

    @post_dump
    def remove_skip_values(self, data, **kwargs):
        """
        Remove None values from the serialization result
        :param data: The input data (dict). Already serialized, but may contain None values
        :param kwargs: Required by overwrite
        :return: Transformed data with None values removed
        """
        return {
            key: value for key, value in data.items()
            if isinstance(value, list) or isinstance(value, dict) or value not in self.SKIP_VALUES
        }


class Model:
    """Represents a basic API Resource model"""

    @classmethod
    def get_class_instance(cls):
        return cls

    def to_json(self):
        return marshmallow_dataclass.class_schema(self.get_class_instance(), base_schema=CamelCaseSchema)().dump(self)

    def __init__(self, **kwargs):
        """
        Default constructor that sets all kwargs as internal properties
        :param kwargs: dictionary of parameters that this class must support
        """
        self.__dict__ = kwargs

    @classmethod
    def from_json(cls, data: dict) -> 'Model':
        """
        Create a new object from a given JSON.
        :param data: The json data
        :return: New instance of the class
        """
        return marshmallow_dataclass.class_schema(cls, base_schema=CamelCaseSchema)().load(data)



import json
from typing import Optional

import requests as requests
from requests import auth, models

from hostingde.exceptions import ClientException


class HostingDeAuth(auth.AuthBase):
    """
    A Auth injector for the hosting.de API. Injects the token and (optionally) the account into the request body.
    """

    def __init__(self, token: str, token_field: str = 'authToken', account_id: str = None):
        super().__init__()
        self.token = token
        self.token_field = token_field
        self.account_id = account_id

    def __call__(self, r: models.PreparedRequest) -> models.PreparedRequest:
        """
        Rewrite the request to include the auth token.

        :param r: A prepared request
        :return: A modified request that contains the authorization
        """

        # Always inject the token
        if r.body is None:
            request = {}
        else:
            request = json.loads(r.body.decode('utf-8'))

        request[self.token_field] = self.token

        if self.account_id is not None:
            request['ownerAccountId'] = self.account_id

        r.body = json.dumps(request).encode('utf-8')
        return r


class HostingDeSession(requests.Session):
    """
    Custom session implementation contains the Hosting.de authorization implementation
    """

    def __init__(self):
        super().__init__()
        self.base_uri: Optional[str] = None

    def build_path(self, *args, **kwargs):
        """
        Build the url path to the resource queried.

        :param args: A path to the resource
        :param kwargs: Optionally, you can provide the base_uri to override the default implementation
        :return:
        """
        uri = [kwargs.get('base_uri') or self.base_uri]

        if uri[0] is None:
            raise ClientException('No endpoint URL set.')

        uri.extend(args)
        return "/".join(uri)

    def set_endpoint(self, url: str):
        """
        Set the base URL for this session

        :param url: The base url to use for all requests
        :return:
        """
        self.base_uri = url

    def token_auth(self, token: str):
        """
        Authenticate using a token

        :param token: The token to use for authentication
        :return:
        """
        self.auth = HostingDeAuth(token)

    def set_account_context(self, account_id: str):
        """
        Switch context to subaccount

        :param account_id: The account id of the subaccount.
        :return:
        """
        if isinstance(self.auth, HostingDeAuth):
            self.auth.account_id = account_id
        else:
            raise ClientException('Subaccount could not be attached to request')

    def get_account_context(self):
        """
        Get the current account used for requests.

        :return: The account id
        """
        if isinstance(self.auth, HostingDeAuth):
            return self.auth.account_id
        else:
            raise ClientException('Subaccount could not be attached to request')



import json.decoder
from contextlib import contextmanager
from typing import Optional, Type

from requests import Response

from hostingde.exceptions import ApiException, ClientException
from hostingde.model import Model
from hostingde.model.filter import FilterElement
from hostingde.model.sort import SortConfiguration
from hostingde.session import HostingDeSession


class HostingDeCore:

    def __init__(self, parent: 'HostingDeCore' = None):
        """
        Initialize a new Hosting.de API client.

        :param parent: The parent resource, used to retrieve session information. If not provided, a new session
                       is generated. WARNING: In this case, you must authenticate.
        """
        self.session = getattr(parent, 'session', parent) or self.new_session()

    def _post(self, *args, **kwargs):
        """
        Perform a post operation using your current session information.

        :param args: List parameters for post
        :param kwargs: Keyword parameters for post
        :return: The response of the post call
        """
        return self.session.post(*args, **kwargs)

    def _request(self, url, model: Optional[Model] = None, **kwargs):
        """
        Execute a new request, given an URL and a model. To generate a URL, you can use the _build_url() utility
        method.

        :param url: The URL resource to request
        :param model: The model to pass to the endpoint
        :param kwargs: additional keyword arguments to pass to requests.post()
        :return:
        """
        if model is None:
            response = self._post(url, **kwargs)
        else:
            response = self._post(url, json=model.to_json(), **kwargs)

        # Check if error occurred
        try:
            error_check = response.json()
            if error_check.get('status', 'error') == 'error':
                raise ApiException(error_check)
        except json.decoder.JSONDecodeError:
            raise ClientException('Error while reading response from server. '
                                  'Is your endpoint configured correctly?')

        return response

    def _instance(self, instance_type: Type[Model], data: dict):
        """
        Reconstructs a model from the passed in result.

        :param instance_type: The instance type to be reconstructed.
        :param data: The data used to reconstruct the model
        :return: The parsed model
        """
        return instance_type.from_json(data)

    def _bool(self, response: Response):
        """
        Converts a response to a boolean. True, if the request succeeded, otherwise False. Note that a status of
        'pending' also returns False. You may want to use the _async method to wait for the job to complete.

        :param response: The response from the backend.
        :return:
        """
        return response.json().get('status') == 'success'

    def _build_uri(self, service: str, method: str) -> str:
        """
        Build a URL for the given service and method.

        :param service: The service of the method, e.g. 'dns'.
        :param method: The method endpoint for the result.
        :return: The finished URL for the resource
        """
        return self.session.build_path(service, "v1", "json", method)

    def _iter(self,
              url: str, instance_class: Type[Model],
              filter: Optional[FilterElement] = None,
              limit: Optional[int] = None,
              sort: Optional[SortConfiguration] = None):
        """
        Use the generic filtering and sorting API to paginate over results.

        :param url: The URL of the resources to paginate.
        :param instance_class: The expected response container
        :param filter: The filter to be applied to the query
        :param limit: The maximum number of items retrieved per call
        :param sort: The sorting of the resulting list
        :return: The iterator for the resultset
        """
        from hostingde.paginator import HostingDePaginator
        return HostingDePaginator(self, instance_class, url, filter=filter, limit=limit, sort=sort)

    def login(self, url: str, token: str) -> None:
        """
        Perform a login.

        :param url: The URL of the endpoint
        :param token: The token to authorize requests
        """
        self.session.base_uri = url
        self.session.token_auth(token)

    def set_account_context(self, account_id: str):
        """
        Sets the account context for this client.

        :param account_id: The account id
        :return:
        """
        self.session.set_account_context(account_id)

    @contextmanager
    def switch_account_context(self, account_id: str):
        """
        Temporarily switch the account context for this client. After the context guard closes, the context is reset to
        the account that was used prior to the guard.

        :param account_id: The account id to switch to
        :return:
        """
        old_id: str = self.session.get_account_context()

        self.session.set_account_context(account_id)

        yield

        self.session.set_account_context(old_id)

    @staticmethod
    def new_session():
        """
        Generates a new, unauthenticated session

        :return: The new session
        """
        return HostingDeSession()



from dataclasses import dataclass, field
from typing import List, Optional

from hostingde.model import Model
from hostingde.model.record import Record
from hostingde.model.zone_config import ZoneConfig


@dataclass
class UpdateZoneRequest(Model):

    zone_config: ZoneConfig
    records_to_add: Optional[List[Record]] = field(default_factory=list)
    records_to_modify: Optional[List[Record]] = field(default_factory=list)
    records_to_delete: Optional[List[Record]] = field(default_factory=list)



from hostingde.account.account import AccountClient
from hostingde.dns.dns import DnsClient
from hostingde.hostingde import HostingDeCore


class HostingDeClient(HostingDeCore):
    """
    The main client.

    Separates the different problem domains by building clients in this class attributes.
    """

    def __init__(self):
        super().__init__()
        self.dns: DnsClient = DnsClient(self)
        self.account: AccountClient = AccountClient(self)



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from marshmallow_enum import EnumField

from hostingde.model import Model


class RecordType(Enum):
    """
    Available relations for conditional filters.
    """
    A = 'A'
    AAAA = 'AAAA'
    ALIAS = 'ALIAS'
    CAA = 'CAA'
    CERT = 'CERT'
    CNAME = 'CNAME'
    DNSKEY = 'DNSKEY'
    DS = 'DS'
    MX = 'MX'
    NS = 'NS'
    NSEC = 'NSEC'
    NSEC3 = 'NSEC3'
    NSEC3PARAM = 'NSEC3PARAM'
    NULLMX = 'NULLMX'
    OPENPGPKEY = 'OPENPGPKEY'
    PTR = 'PTR'
    RRSIG = 'RRSIG'
    SRV = 'SRV'
    SOA = 'SOA'
    SSHFP = 'SSHFP'
    TLSA = 'TLSA'
    TXT = 'TXT'


@dataclass
class Record(Model):
    """The DNS Record object is part of a zone. It is used to manage DNS resource records."""

    id: Optional[str]
    zone_id: Optional[str]
    record_template_id: Optional[str]
    name: Optional[str]
    content: Optional[str]
    ttl: Optional[int]
    priority: Optional[int]
    last_change_date: Optional[str]
    type: Optional[RecordType] = EnumField(RecordType)

    def __init__(self,
                 id: Optional[str] = None,
                 zone_id: Optional[str] = None,
                 record_template_id: Optional[str] = None,
                 name: Optional[str] = None,
                 type: Optional[RecordType] = None,
                 content: Optional[str] = None,
                 ttl: Optional[int] = None,
                 priority: Optional[int] = None,
                 last_change_date: Optional[str] = None):
        """
        The DNS Record object is part of a zone. It is used to manage DNS resource records.

        :param id:                 Record ID. Ignored in create zone requzone_config.pyests. Either id or zoneId, name,
                                   type, and content are required in all other requests.

        :param zone_id:            ID of zone that the record belongs to.

        :param record_template_id: ID of record template the record is tied to. If empty, record has to be managed
                                   manually. If tied to record template, record will be removed or updated whenever
                                   record template is removed or updated.

        :param name:               Name of the record. Example: mail.example.com. Always required in create requests and
                                   in all other requests if no id is provided.

        :param type:
                                   Type of the record. Valid types are A, AAAA, ALIAS, CAA, CERT, CNAME, DNSKEY, DS, MX,
                                   NS, NSEC, NSEC3, NSEC3PARAM, NULLMX, OPENPGPKEY, PTR, RRSIG, SRV, SSHFP, TLSA, and
                                   TXT. Always required in create requests and in all other requests if no id is
                                   provided.

        :param content:            Content of the record. Always required in create requests and in all other requests
                                   if no id is provided.

        :param ttl:                TTL of the record in seconds. Minimum value: 60. Maximum value: 31556926 (one year).
                                   Exceeding the maximum or undercutting the minimum value will abort the request and
                                   result in an error.

        :param priority:           Priority of record. Only relevant if type is MX or SRV. Must be positive.

        :param last_change_date:   Date and time of last record modification
        """
        super().__init__()
        self.id = id
        self.zone_id = zone_id
        self.record_template_id = record_template_id
        self.name = name
        self.type = type
        self.content = content
        self.ttl = ttl
        self.priority = priority
        self.last_change_date = last_change_date

    def __eq__(self, other):
        if not isinstance(other, Record):
            return False

        return self.name == other.name and self.type == other.type and self.content == other.content

    def __hash__(self):
        return hash(f'Name:{self.name}Type:{self.type.value if self.type else "No type"}Content:{self.content}')

    def __str__(self):
        return f"{self.name} {self.type.value} {self.content}"

    @staticmethod
    def create_new_record(name: str, type: RecordType, content: str, ttl: Optional[int] = 86400,
                          priority: Optional[str] = None):
        return Record(
            name=name,
            type=type,
            content=content,
            ttl=ttl,
            priority=priority
        )



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from marshmallow_enum import EnumField

from hostingde.model import Model


class JobStatus(Enum):
    inProgress = 'inProgress'
    successful = 'successful'
    failed = 'failed'
    canceled = 'canceled'
    new = 'new'
    support = 'support'


@dataclass
class Job(Model):
    account_id: Optional[str]
    action: Optional[str]
    add_date: Optional[str]
    display_name: Optional[str]
    id: Optional[str]
    last_change_date: Optional[str]
    object_id: Optional[str]
    object_type: Optional[str]
    parent_job_id: Optional[str]
    status: JobStatus = EnumField(JobStatus)

    def __init__(self, account_id: str = None, action: str = None, add_date: str = None, display_name: str = None,
                 id: str = None, last_change_date: str = None,
                 object_id: str = None, object_type: str = None, parent_job_id: str = None, status: JobStatus = None,
                 **kwargs):
        """
        Represents a job.

        :param account_id: The account id that this job belongs to.
        :param action: The action of the job, e.g. 'create', 'delete'
        :param add_date: When the job was first started.
        :param display_name: The readable name of the resource affected
        :param id: The unique job id.
        :param last_change_date: When the job was last updated.
        :param object_id: The id of the object affected by the job
        :param object_type: The type of the object, e.g. 'Zone', 'Record'
        :param parent_job_id: If the job was started by another job
        :param status: The current status of the job, e.g. 'inProgress', 'successfull'
        :param kwargs:
        """
        super().__init__()
        self.account_id = account_id
        self.action = action
        self.add_date = add_date
        self.display_name = display_name
        self.id = id
        self.last_change_date = last_change_date
        self.object_id = object_id
        self.object_type = object_type
        self.parent_job_id = parent_job_id
        self.status = status

    def __str__(self):
        return f"{self.display_name} -> {self.status.value}"



from dataclasses import dataclass, field
from typing import List

from hostingde.model import Model
from hostingde.model.record import Record
from hostingde.model.zone_config import ZoneConfig


@dataclass
class Zone(Model):
    """
    A generic zone object used for responses.
    """

    zone_config: ZoneConfig
    records: List[Record] = field(default_factory=list)

    def __str__(self):
        return f"Name: {self.zone_config.name}; Type: {self.zone_config.type.value}; Status: {self.zone_config.status}"



from dataclasses import dataclass
from enum import Enum

from marshmallow_enum import EnumField

from hostingde.model import Model


class SortOrder(Enum):
    ASC = 'ASC'
    DESC = 'DESC'


@dataclass
class SortConfiguration(Model):
    """
    Adds sorting configuration to the endpoint
    """

    field: str
    order: SortOrder = EnumField(SortOrder)

    def __init__(self, field: str, order: SortOrder):
        """
        The field you want to order on.

        :param field: The field name
        :param order: The ordering of the results. Either 'ASC' or 'DESC'
        """
        super().__init__()
        self.field: str = field
        self.order: SortOrder = order



from dataclasses import dataclass
from typing import Optional

from hostingde.model import Model


@dataclass
class Account(Model):
    """
        Name
        Name of the Account

        id
        ID of this Account.
    """

    name: Optional[str]
    id: Optional[str]

    def __init__(self, name: Optional[str], id: Optional[str]):
        """
        The Account object a object.
        :param name: Namw of this Account

        :param id: ID of this Account.
        """
        self.id: Optional[str] = id
        self.name: Optional[str] = name



from abc import ABC, abstractmethod
from enum import Enum
from typing import Optional, List, Union


class FilterCompilationException(Exception):
    pass


class FilterElement(ABC):

    @abstractmethod
    def to_filter_object(self):
        """
        Converts the filter element into a filter object for API consumption.
        :return: The JSON object consumed by the API.
        """
        pass


class FilterConditionRelation(Enum):
    """
    Available relations for conditional filters.
    """
    EQUAL = 'equal'
    UNEQUAL = 'unequal'
    GREATER = 'greater'
    LESS = 'less'
    GREATER_EQUAL = 'greaterEqual'
    LESS_EQUAL = 'lessEqual'


class FilterCondition(FilterElement):
    """
     The result will match to this condition. You can find lists or available and valid fields throughout the API.
     Please refer to specific documentation sections for the respective finding methods. Field names are case
     insensitive.
    """

    def to_filter_object(self):
        """
        Converts the filter element into a filter object for API consumption.
        :return: The JSON object consumed by the API.
        :raise FilterCompilationException: If the filter did not contain the necessary fields.
        """
        if self.field is not None and self.value is not None:
            return {
                "field": self.field,
                "value": self.value,
                "relation": str(self.relation.value)
            }
        else:
            raise FilterCompilationException(f'Value for field "{self.field}" was not specified.')

    def __init__(self, field: str,
                 value: Optional[str] = None,
                 relation: Optional[FilterConditionRelation] = None):
        """
        In its simplest form, the filter parameter takes a field and a value parameter. The field element is restricted
        to a list of field names which vary from listing to listing.

        :param field: refers to the field you want to filter
        :param value: is the argument for that field. By default, the value is a case-insensitive exact match. An
                      asterisk (*) can be used to match an arbitrary number of characters (including zero characters).
        :param relation: The relation element specifies the comparison performed on field and the specific value. It is
                         an optional element and defaults to ‘equal’ if not set. Please see the table below for further
                         details and explanations.

        :example:

        >>> f = FilterCondition('someField').eq('you') & FilterCondition('otherField').ne('me')
        >>> print(f.to_filter_object())
        {'subFilterConnective': 'and', 'subFilter': [{'field': 'someField', 'value': 'you', 'relation': 'equal'},
        {'field': 'otherField', 'value': 'me', 'relation': 'unequal'}]}
        """
        self.field: str = field
        self.value: str = value
        self.relation: Optional[FilterConditionRelation] = relation

    def eq(self, other: Union[str, int, float]):
        """
        Override == setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.EQUAL
        return self

    def ne(self, other: Union[str, int, float]):
        """
        Override != setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.UNEQUAL
        return self

    def lt(self, other: Union[str, int, float]):
        """
        Override < setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.LESS
        return self

    def le(self, other: Union[str, int, float]):
        """
        Override <= setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.LESS_EQUAL
        return self

    def gt(self, other: Union[str, int, float]):
        """
        Override > setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.GREATER
        return self

    def ge(self, other: Union[str, int, float]):
        """
        Override >= setter for value.
        :param other: The value to be set
        :return:
        """
        self.value = other
        self.relation = FilterConditionRelation.GREATER_EQUAL
        return self

    def startswith(self, begin: str):
        """
        Filter for entries starting with a specific pattern
        :param begin: The begin filter
        :return:
        """
        self.value = f"{begin}*"
        self.relation = FilterConditionRelation.EQUAL
        return self

    def contains(self, value: str):
        """
        Filter condition with contains semantic. Filters for entities that contain the given value
        :param value: The contain filter value
        :return:
        """
        self.value = f"*{value}*"
        self.relation = FilterConditionRelation.EQUAL
        return self

    def __and__(self, other: FilterElement) -> 'FilterChain':
        """
        Dynamically construct a 'and' filter chain object using binary operations.
        :param other: The other filter element to chain to this element.
        :return:
        """
        if isinstance(other, FilterCondition):
            return FilterChain(FilterChainConnective.AND).add_filter(self).add_filter(other)
        elif isinstance(other, FilterChain):
            if other.connective == FilterChainConnective.AND:
                return other.add_filter(self)
            elif other.connective == FilterChainConnective.OR:
                return FilterChain(FilterChainConnective.AND).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')
        else:
            raise FilterCompilationException('Unknown filter element type')

    def __or__(self, other: FilterElement) -> 'FilterChain':
        """
        Dynamically construct a 'or' filter chain object using binary operations.
        :param other: The other filter element to chain to this element.
        :return: A FilterChain implementation for the given logic.
        """
        if isinstance(other, FilterCondition):
            return FilterChain(FilterChainConnective.OR).add_filter(self).add_filter(other)
        elif isinstance(other, FilterChain):
            if other.connective == FilterChainConnective.OR:
                return other.add_filter(self)
            elif other.connective == FilterChainConnective.AND:
                return FilterChain(FilterChainConnective.OR).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')
        else:
            raise FilterCompilationException('Unknown filter element type')


class FilterChainConnective(Enum):
    """
    Available connectives for chain filters.
    """
    AND = 'and'
    OR = 'or'


class FilterChain(FilterElement):
    """
    Construct a filter chain containing multiple filter conditions. Supports a connective relation to be set. Supported
    are AND and OR connectives.
    """

    def to_filter_object(self):
        """
        Recursively builds the filter objects to be constructed.
        :return:
        """

        if len(self.filters) == 0:
            raise FilterCompilationException('ChainFilter has no filters attached.')

        return {
            "subFilterConnective": self.connective.value,
            "subFilter": [f.to_filter_object() for f in self.filters]
        }

    def __init__(self, connective: FilterChainConnective):
        """
        Start a new filter chain. You will have to set the connective at the very least.
        :param connective: The connective to use for this chain. Either 'and' or 'or'.
        """
        self.connective: FilterChainConnective = connective
        self.filters: List[FilterElement] = []

    def add_filter(self, filter_element: FilterElement, front: bool = False) -> 'FilterChain':
        """
        Add a new filter element into this chain.
        :param filter_element: The element to add to the filter chain.
        :param front: Insert the element in the front of the array
        :return: This object in order to chain add_filter operations together
        """
        if front:
            self.filters.insert(0, filter_element)
        else:
            self.filters.append(filter_element)

        return self

    def __and__(self, other: FilterElement) -> 'FilterChain':
        """
        Dynamically construct a 'and' filter chain object using binary operations.
        :param other: The other filter element to chain to this element.
        :return:
        """
        if isinstance(other, FilterCondition):
            if self.connective == FilterChainConnective.AND:
                return self.add_filter(other)
            elif self.connective == FilterChainConnective.OR:
                return FilterChain(FilterChainConnective.AND).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')
        elif isinstance(other, FilterChain):
            if self.connective == FilterChainConnective.AND and other.connective == FilterChainConnective.AND:
                # filters can be merged
                for f in other.filters:
                    self.add_filter(f)
                return self
            elif self.connective == FilterChainConnective.AND and other.connective == FilterChainConnective.OR:
                return self.add_filter(other)
            elif self.connective == FilterChainConnective.OR and other.connective == FilterChainConnective.AND:
                return other.add_filter(self, True)
            elif self.connective == FilterChainConnective.OR and other.connective == FilterChainConnective.OR:
                return FilterChain(FilterChainConnective.AND).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')

    def __or__(self, other: FilterElement) -> 'FilterChain':
        """
        Dynamically construct a 'and' filter chain object using binary operations.
        :param other: The other filter element to chain to this element.
        :return:
        """
        if isinstance(other, FilterCondition):
            if self.connective == FilterChainConnective.OR:
                return self.add_filter(other)
            elif self.connective == FilterChainConnective.AND:
                return FilterChain(FilterChainConnective.OR).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')
        elif isinstance(other, FilterChain):
            if self.connective == FilterChainConnective.OR and other.connective == FilterChainConnective.OR:
                # filters can be merged
                for f in other.filters:
                    self.add_filter(f)
                return self
            elif self.connective == FilterChainConnective.OR and other.connective == FilterChainConnective.AND:
                return self.add_filter(other)
            elif self.connective == FilterChainConnective.AND and other.connective == FilterChainConnective.OR:
                return other.add_filter(self, True)
            elif self.connective == FilterChainConnective.AND and other.connective == FilterChainConnective.AND:
                return FilterChain(FilterChainConnective.OR).add_filter(self).add_filter(other)
            else:
                raise FilterCompilationException('Unknown filter element connective operation')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from setuptools import find_packages, setup


def get_version():
    with open("hostingde/__version__.py") as f:
        for line in f:
            if line.startswith("__version__"):
                return eval(line.split("=")[-1])


with open("README.md", "r") as readme_file:
    readme = readme_file.read()

setup(
    name="python-hostingde",
    version=get_version(),
    description="Interact with HostingDe API",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Benedict Becker",
    author_email="benedict.becker93@gmail.com",
    license="LGPLv3",
    url="https://github.com/becelot/python-hostingde",
    packages=find_packages(exclude=["docs*", "tests*"]),
    install_requires=[
        "requests>=2.25.1",
        "marshmallow==3.10.0",
        "marshmallow-dataclass==8.3.1",
        "marshmallow-enum==1.5.1",
        "urllib3~=1.26.3",
        "responses~=0.13.1"
    ],
    python_requires=">=3.6.0",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from abc import ABC, abstractmethod
import time
from typing import Optional

from hostingde.model.filter import FilterElement, FilterCondition
from hostingde.model.job import Job
from hostingde.model.sort import SortConfiguration
from hostingde.paginator import HostingDePaginator


class AsynchronousClient(ABC):

    @abstractmethod
    def jobs_find(self,
                  limit: Optional[int] = None,
                  filter: Optional[FilterElement] = None,
                  sort: Optional[SortConfiguration] = None,
                  *args, **kwargs) -> HostingDePaginator[Job]:
        """
        Retrieves a list of Job objects from the generic filtering and sorting API.

        :param limit: The limit of objects to retrieve per call. If not set, defaults to 25.
        :param filter: A filter that is applied to the query
        :param sort: Configuration how results are sorted.
        :return: An iterator that yields ZoneConfig objects.
        """
        pass


class JobWaiter:

    def __init__(self, service: AsynchronousClient, id: str):
        self.service = service
        self.id = id

    def wait(self):
        while True:
            jobs = self.service.jobs_find(
                filter=FilterCondition('jobObjectId').eq(self.id)
                       & FilterCondition('jobStatus').ne('successful')
                       & FilterCondition('jobStatus').ne('failed')
                       & FilterCondition('jobStatus').ne('canceled')
            ).fetchall()

            if len(jobs) == 0:
                break

            time.sleep(1)



from dataclasses import field, dataclass
from typing import TypeVar, Generic, Optional, List, Type

from hostingde.exceptions import ClientException
from hostingde.hostingde import HostingDeCore
from hostingde.model import Model
from hostingde.model.filter import FilterElement
from hostingde.model.sort import SortConfiguration

R = TypeVar('R')


@dataclass
class PaginatedRequest(Model):
    """
    Represents a paginated request against a generic filtering and sorting API
    """

    filter: Optional[dict] = field(default=None)
    limit: Optional[int] = field(default=None)
    page: Optional[int] = field(default=None)
    sort: Optional[SortConfiguration] = field(default=None)


class HostingDePaginator(HostingDeCore, Generic[R]):

    def __init__(self,
                 parent: HostingDeCore,
                 instance_class: Type[Model],
                 url: str,
                 count: Optional[int] = -1,
                 limit: Optional[int] = 25,
                 filter: Optional[FilterElement] = None,
                 sort: Optional[SortConfiguration] = None):
        """
        Construct a new paginator.

        :param parent: The parent to get session details from
        :param instance_class: The instance class to create
        :param url: The URL of the resource to paginate over
        :param count: The total amount of entries to retrieve
        :param limit: The maximum number of entries to retrieve per API call
        :param filter: Filter the results based on a filter expression
        :param sort: Sort the results by a given field
        """
        super().__init__(parent)
        self.current_page = 1
        self.total_pages = -1
        self.results: List[R] = []
        self.limit = limit if limit is not None and limit > 0 else 25
        self.count = -1 if count is None or count <= 0 else count
        self.filter = filter
        self.sort = sort
        self.url = url
        self.count = count
        self.instance_class = instance_class
        self._total_entries = -1

    def __iter__(self):
        """
        This object is a iterator itself.

        :return: Self
        """
        return self

    def _load_next(self):
        # No more results cached, and more available, load new results
        response = self._request(self.url, model=PaginatedRequest(
            filter=self.filter.to_filter_object() if self.filter is not None else None,
            limit=self.limit,
            page=self.current_page,
            sort=self.sort
        ))

        data = response.json().get('response', {})

        # if this was the first call, retrieve the amount of total pages
        if self.total_pages == -1:
            self.total_pages = data.get('totalPages', -1)
            self._total_entries = data.get('totalEntries', -1)

        self.current_page += 1

        # Extract and convert the results
        if len(data.get('data', [])) > 0:
            self.results.extend(map(lambda x: self.instance_class.from_json(x), data.get('data', [])))

    def __next__(self):
        """
        Get the next result.

        :return: The next entry in the list
        """
        # Still requested more?
        if self.count == 0:
            raise StopIteration

        # If data is still available
        if len(self.results) > 0:
            self.count -= 1
            return self.results.pop(0)

        # Are there more pages to be retrieved?
        if self.total_pages < self.current_page and self.total_pages != -1:
            raise StopIteration

        self._load_next()

        # Extract and convert the results
        if len(self.results) > 0:
            self.count -= 1
            return self.results.pop(0)

        raise StopIteration

    def fetchall(self) -> List[R]:
        """
        Do not cache results, load all into memory.

        :return: The list of objects that this paginator generates
        """
        return list(self)

    def fetchone(self) -> R:
        """
        Only fetch a single resource from the API.

        :return: A single object returned from the API
        """
        try:
            return self.__next__()
        except StopIteration:
            raise ClientException('No entities found for given filter!')

    def __len__(self):
        """
        Length of items available
        :return: Number of entities
        """

        # If not yet loaded, prefetch data
        if self._total_entries == -1:
            self._load_next()

        return self._total_entries if self._total_entries >= 0 else 0



from hostingde.client import HostingDeClient
from hostingde.exceptions import ClientException


def login(base_url: str, token: str):
    """
    Entry point for the client. Builds and logs the user in.

    :param base_url: The base url for the backend, e.g. 'https://demo.routing.net/api'
    :param token: The token for authorization
    :return:
    """

    if base_url is None or base_url.strip() == "":
        raise ClientException('No base url set!')

    client = HostingDeClient()
    client.login(base_url, token)
    return client



from dataclasses import dataclass
from typing import Optional, List

from marshmallow_enum import EnumField

from hostingde.exceptions import ClientException
from hostingde.model import Model
from hostingde.model.soa_values import SoaValues
from enum import Enum


class ZoneConfigType(Enum):
    """
    Available relations for conditional filters.
    """
    NATIVE = 'NATIVE'
    MASTER = 'MASTER'
    SLAVE = 'SLAVE'


@dataclass
class ZoneConfig(Model):
    """
    The ZoneConfig object defines a zone.

    type
        Valid types are NATIVE, MASTER, and SLAVE

    id
      ID of this zoneConfig. Ignored in zone create requests. Either id or name is required in all other
      requests.

    status
        Current status of the Zone

    name
        The zone name in ACE format. name is always required in zone create requests and all other requests
        if no id is provided. While you may use this property for a Unicode zone name, the responses will
        always contain the ACE encoded zone.

    name_unicode
        The zone name in Unicode. If it is left empty it will be filled automatically based on name.

    master_ip
        A valid IPv4 or IPv6 must be provided if the type is SLAVE. If the type is NATIVE or MASTER
        the field must be empty.

    email_address
        The hostmaster email address. Only relevant if the type is NATIVE or MASTER. If the field
        is left empty, the default is hostmaster@name.

    zone_transfer_whitelist
        List of IP addresses whitelisted for zone transfers. Only allowed if the type is
        MASTER. Must contain valid IPv4 or IPv6 addresses.

    last_change_date
        Date and time the zone was last modified

    soa_values
        Values for the SOA record of the zone. If it is left empty an SOA record with default values
        will be created.
    """

    id: Optional[str]
    account_id: Optional[str]
    status: Optional[str]
    name: Optional[str]
    name_unicode: Optional[str]
    dns_sec_mode: Optional[str]
    master_ip: Optional[str]
    email_address: Optional[str]
    zone_transfer_whitelist: Optional[List[str]]
    last_change_date: Optional[str]
    soa_values: Optional[SoaValues]
    type: ZoneConfigType = EnumField(ZoneConfigType)

    def __init__(self, type: ZoneConfigType, id: Optional[str] = None, account_id: Optional[str] = None,
                 status: Optional[str] = None, name: Optional[str] = None,
                 name_unicode: Optional[str] = None, master_ip: Optional[str] = None,
                 email_address: Optional[str] = None, dns_sec_mode: Optional[str] = None,
                 zone_transfer_whitelist: Optional[List[str]] = None, last_change_date: Optional[str] = None,
                 soa_values: Optional[SoaValues] = None, **kwargs):
        """
        The ZoneConfig object defines a zone.

        :param type: Valid types are NATIVE, MASTER, and SLAVE

        :param id: ID of this zoneConfig. Ignored in zone create requests. Either id or name is required in all other
                   requests.

        :param account_id: ID of the account that manages the zone. This property is never used in requests.

        :param status: Current status of the Zone

        :param name: The zone name in ACE format. name is always required in zone create requests and all other requests
                     if no id is provided. While you may use this property for a Unicode zone name, the responses will
                     always contain the ACE encoded zone.

        :param name_unicode: The zone name in Unicode. If it is left empty it will be filled automatically based on
                             name.

        :param master_ip: A valid IPv4 or IPv6 must be provided if the type is SLAVE. If the type is NATIVE or MASTER
                          the field must be empty.

        :param email_address: The hostmaster email address. Only relevant if the type is NATIVE or MASTER. If the field
                              is left empty, the default is hostmaster@name.

        :param zone_transfer_whitelist: List of IP addresses whitelisted for zone transfers. Only allowed if the type is
                                        MASTER. Must contain valid IPv4 or IPv6 addresses.

        :param last_change_date: Date and time the zone was last modified

        :param soa_values: Values for the SOA record of the zone. If it is left empty an SOA record with default values
                           will be created.
        """
        super().__init__()
        self.id: Optional[str] = id
        self.account_id: Optional[str] = account_id
        self.status: Optional[str] = status
        self.name: Optional[str] = name
        self.name_unicode: Optional[str] = name_unicode
        self.master_ip: Optional[str] = master_ip
        self.type: Optional[ZoneConfigType] = type
        self.email_address: Optional[str] = email_address
        self.zone_transfer_whitelist: Optional[List[str]] = zone_transfer_whitelist
        self.last_change_date: Optional[str] = last_change_date
        self.soa_values: Optional[SoaValues] = soa_values
        self.dns_sec_mode = dns_sec_mode

    @staticmethod
    def create_new_zone_config(name: str, type: ZoneConfigType, master_ip: Optional[str] = None,
                               id: Optional[str] = None, email_address: str = None, soa_values: SoaValues = None,
                               name_unicode: str = None, zone_transfer_whitelist: List[str] = None):
        if master_ip is not None and type != ZoneConfigType.SLAVE:
            raise ClientException("MasterIp can only be set if the type is SLAVE")

        if zone_transfer_whitelist is not None and type != ZoneConfigType.MASTER:
            raise ClientException("ZoneTransferWhiteList can only be used if the type is MASTER")

        if name is None:
            raise ClientException("No name for the zone was specified!")

        if type is None:
            raise ClientException("You must specify a zone type!")

        return ZoneConfig(
            id=id,
            name=name,
            master_ip=master_ip,
            name_unicode=name_unicode,
            email_address=email_address,
            soa_values=soa_values,
            type=type,
            zone_transfer_whitelist=zone_transfer_whitelist
        )

    def __str__(self):
        return f"ZoneConfig for {self.name}"
